#!/usr/bin/env python
#---------------------------------------------------------------
# Project         : Mandrake Linux
# Module          : rpm-rebuilder
# File            : files-to-requires.py
# Version         : $Id: files-to-requires.py,v 1.4 2003/05/06 08:09:01 flepied Exp $
# Author          : Frederic Lepied
# Created On      : Sat Sep 30 14:17:11 2000
#---------------------------------------------------------------

import rpm
import re
import sys
import os

try:
    if rpm.RPMTAG_SOURCEPACKAGE:
        v42=1
except AttributeError:
    v42=0

verbose=(len(sys.argv) > 1)
    
IMPLICIT_DEPS = (
    'bash',
    'binutils',
    'make',
    'gcc',
    'glibc',
    'bzip2',
    'tar',
    'rpm',
    'patch',
    'glibc-devel',
    'spec-helper',
    # basesystem is bellow
    'setup',
    'filesystem',
    'basesystem',
    'sed',
    'initscripts',
    'console-tools',
    'utempter',
    'ldconfig',
    'chkconfig',
    'ntsysv',
    'SysVinit',
    'bdflush',
    'crontabs',
    'dev',
    'e2fsprogs',
    'etcskel',
    'fileutils',
    'findutils',
    'getty_ps',
    'grep',
    'gzip',
    'hdparm',
    'info',
    'kernel',
    'less',
    'logrotate',
    'losetup',
    'man',
    'mingetty',
    'modutils',
    'mount',
    'net-tools',
    'passwd',
    'procmail',
    'procps',
    'psmisc',
    'mandrake-release',
    'rootfiles',
    'rpm',
    'sash',
    'ash',
    'setserial',
    'shadow-utils',
    'sh-utils',
    'stat',
    'sysklogd',
    'tar',
    'termcap',
    'textutils',
    'time',
    'tmpwatch',
    'util-linux',
    'vim',
    'vixie-cron',
    'which',
    'perl-base',
    'msec',
    'lilo',
    'grub',
    'mkbootdisk',
    'isapnptools'
    )

build_requires=[]
pkg2files={}

def add_req(pkg):
    if pkg not in IMPLICIT_DEPS:
        if pkg not in build_requires:
            build_requires.append(pkg)
            if verbose: pkg2files[pkg]=[file]
        else:
            if verbose: pkg2files[pkg].append(file)
    
if not v42:
    db=rpm.opendb()

for line in sys.stdin.readlines():
    file=line[:-1]
    if not os.path.isdir(file):
        if v42:
            ts=rpm.TransactionSet()
            mi=ts.dbMatch(rpm.RPMTAG_BASENAMES, file)
            while mi:
                h=mi.next()
                if not h:
                    break
                add_req(h[rpm.RPMTAG_NAME])
        else:
            try:
                for idx in db.findbyfile(file):
                    add_req(db[idx][rpm.RPMTAG_NAME])
            except rpm.error:
                pass
            del db

for d in build_requires:
    print d

if verbose:
    for p in pkg2files.keys():
        sys.stderr.write("%s: %s\n" % (p, pkg2files[p]))

# files-to-requires.py ends here
