#!/bin/sh
#---------------------------------------------------------------
# Project         : Mandriva Linux
# Module          : rpm-rebuilder
# File            : install-chroot-tar.sh
# Version         : $Id: install-chroot-tar.sh,v 1.1 2005/11/04 23:50:49 flepied Exp $
# Author          : Frederic Lepied
# Created On      : Fri Nov  4 14:15:13 2005
# Purpose         : install and cache a set of packages.
#---------------------------------------------------------------

set -e

if [ $# -lt 6 ]; then
    echo "usage: $0 <media> <repository> <file> <dest> <uid> <pkg> [<pkg2>...]" 1>&2
    exit 1
fi

MEDIA=$1
REPOS=$2
FILE=$3
DEST=$4
CHROOT_UID=$5
shift 5
RPMS="$@"

function build_chroot() {
    mkdir -p $DEST/dev
    mkdir -p $DEST/etc/sysconfig $DEST/dev/pts
    touch $DEST/etc/ld.so.conf $DEST/etc/mtab
    mknod  $DEST/dev/null c 1 3
    chmod a+rw $DEST/dev/null
    urpmi -q --media $MEDIA --auto --no-verify-rpm --root $DEST $RPMS
    chroot $DEST rpm -qa --qf '%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}.rpm\n' > $DEST/var/log/qa
    chroot $DEST adduser -o --uid $CHROOT_UID builder
    mkdir -p $DEST/home/builder/rpm/{RPMS,BUILD,SPECS,SRPMS,SOURCES}
    chown -R $CHROOT_UID $DEST/home/builder
    tar cf $FILE -C $DEST .
}

rm -rf $DEST

if [ ! -f $FILE ]; then
    build_chroot
else
    mkdir -p $DEST
    tar xf $FILE -C $DEST
    for f in `cat $DEST/var/log/qa`; do
	if [ ! -f $REPOS/$f ]; then
	    echo "$f has changed rebuilding chroot tar ball"
	    rm -rf $DEST
	    build_chroot
	fi
    done
fi

# install-chroot-tar.sh ends here
