#---------------------------------------------------------------
# Project         : Mandriva Linux
# Module          : rpm-rebuilder
# File            : specfile.py
# Version         : $Id: specfile.py,v 1.8 2005/09/10 21:27:38 flepied Exp $
# Author          : Frederic Lepied
# Created On      : Sun Jul 30 23:45:19 2000
# Purpose         : library to read and manipulate an rpm specfile.
#---------------------------------------------------------------

import sys
import string
import re
import time
import commands

PKG_RGX="%package\s+(-n)?\s*"
define_regex=re.compile("%{?([^}]+)}?")
package_regex=re.compile(PKG_RGX+"([^ \t\n]+)")
source_regex=re.compile("Source[0-9]*\s*:\s*(.+)\s*$")
shell_regex=re.compile('(.*?)%\((.+?)\)(.*)')
macro_regex=re.compile("(.*?)%{(.+?)}(.*)")
macro2_regex=re.compile("(.*?)%([^-./ ]+)(.*)")
mkrel_regex=re.compile('\s*%mkrel\s+(\d+)')

class specfile:
    def __init__(self, filename):
        self.lastdefine=None
        self.lastline=None
        self._packages=None
        self.use_mkrel=False
        self.sources=None
        fd = open(filename, "r")
        in_preamble=1
        self.lines = fd.readlines()
        fd.close()

    def subPackages(self):
        if self._packages == None:
            self._packages={}
            for idx in range(0,len(self.lines)):
                res=package_regex.search(self.lines[idx])
                if res:
                    self._packages[res.group(2) or res.group(1)]=idx
        return self._packages.keys()
    
    def lookupField(self, name, start=0, end=None):
        length=len(name)
        if not end:
            end=len(self.lines)
        lgth=end-start
        for idx in range(start, lgth):
            l=self.lines[idx]
            line=string.strip(l)
            if name.upper() == line[0:length].upper():
                str=string.strip(line[length+1:])
                self.lastline=l
                return self.expand_macros(str)
        return None
    
    def lookupDefine(self, name):
        regex=re.compile("%define\s+"+ name + "\s+([^#]+?)\s*($|#)")
        for l in self.lines:
            res=regex.search(l)
            if res:
                self.lastline=l
                return res.group(1)
        if name == 'name':
            return self.lookupField('Name')
        if name == 'version':
            return self.lookupField('Version')
        if name == 'release':
            return self.lookupField('Release')
        if name == 'nil':
            return ''
        return None
    
    def dump(self):
        for l in self.lines:
            sys.stdout.write(l)
        
    def _incRelease(self, rel):
        # split the release to handle 0.15mdk relase
        list=string.split(rel, '.')
        # extract the number from 15mdk
        last=list[-1]
        for idx in range(0, len(last)-1):
            if not last[idx] in string.digits:
                break
        newrel=string.atoi(last[0:idx])+1
        # rebuild the release string
        list[-1]=repr(newrel)+last[idx:]
        return string.join(list, '.')
        
    def incrementRelease(self, email, changelog):
        ver=self.lookupField('Version')
        rel=self.lookupField('Release')
        rel_line=self.lastline
        if not rel or not ver:
            return None
        newrel=self._incRelease(rel)
        # replace the new strings in the spec
        for idx in range(0, len(self.lines) - 1):
            l=self.lines[idx]
            if l == rel_line:
                if self.use_mkrel:
                    self.lines[idx]=string.replace(l, rel[:-3], newrel[:-3])
                else:
                    self.lines[idx]=string.replace(l, rel, newrel)
            elif l[:10] == '%changelog':
                self.lines[idx]="%%changelog\n* %s %s %s-%s\n- %s\n\n" \
                                 % (time.strftime("%a %b %d %Y", time.gmtime(time.time())),
                                    email, ver, newrel, changelog)
        # try to see if there are sub-packages with an
        # independant release numbering
        pkgs=self.subPackages()
        length=len(pkgs)
        for idx in range(0, length):
            if idx != length - 1:
                rel=self.lookupField('Release', self._packages[pkgs[idx]],
                                     self._packages[pkgs[idx+1]])
            else:
                rel=self.lookupField('Release', self._packages[pkgs[idx]])
            if rel:
                newrel=self._incRelease(rel)
                # replace the new strings in the spec
                for loop in range(0, len(self.lines) - 1):
                    l=self.lines[loop]
                    if l == self.lastline:
                        self.lines[loop]=string.replace(l, rel, newrel)

    def replaceField(self, name, value):
        define=self.lookupField(name)
        if define:
            for idx in range(0, len(self.lines)):
                if self.lastline == self.lines[idx]:
                    self.lines[idx]=string.replace(self.lastline, define, value)
                    break
        # the field doesn't exist, insert it after the Group tag
        else:
            define=self.lookupField('Group')
            if define:
                for idx in range(0, len(self.lines)):
                    if self.lastline == self.lines[idx]:
                        self.lines[idx]="%s%s: %s\n" % (self.lastline, name, value)
                        break
            # no Group ! insert after first line
            else:
                self.lines[0]="%s%s: %s\n" % (self.lines[0], name, value)

    def expand_macros(self, str):
        #print 'expand_macros', str
        # handle %mkrel
        res=mkrel_regex.search(str)
        if res:
            self.use_mkrel=True
            return res.group(1) + 'mdk'
        # handle %( ) constructs
        res = shell_regex.search(str)
        if res:
            #print 'group', res.group(2)
            out = commands.getoutput(self.expand_macros(res.group(2)))
            #print 'out', out
            return res.group(1) + out + res.group(3)
        res = macro_regex.search(str) or macro2_regex.search(str)
        if res:
            expand = self.lookupDefine(res.group(2))
            if expand == None:
                print "cannot expand", res.group(2)
                return str
            return self.expand_macros(res.group(1) + expand + res.group(3))
        return str
    
    def get_sources(self):
        if self.sources != None:
            return self.sources
        self.sources = []
        for line in self.lines:
            res = source_regex.search(line)
            if res:
                #print res.group(1)
                self.sources.append(self.expand_macros(res.group(1)))
        return self.sources
    
# specfile.py ends here
